;        gi-mc-list
;        Machine code routines for gamesindex

chardata EQU FF9A,65434      ;array name - .$()            4
tapedata EQU 4000,16384      ;tape header area             17
save-IY  EQU FFA2,65442      ;saved IY                     } use spare
matched  EQU FFAC,65452      ;0=not found  1=found         } bytes in
element  EQU FFAD,65453      ;element count for x$         } header
namedata EQU FFAF,65455      ;curr/prev name in array      40+40

;NOTE: tapedata was moved to 16384, as the top 16k seems to be paged out
;      during tape access, so validation was failing


;        Extend array

extend-x LD A,78             ;code "x"
         JR extend
extend-z LD A,7A             ;code "z"
extend   CALL find-var
         LD BC,block         ;bytes required
         LD L,(IX+01)
         LD H,(IX+02)
         PUSH HL             ;save array size
         ADD HL,BC           ;update array size
         LD (IX+01),L
         LD (IX+02),H
         LD L,(IX+06)
         LD H,(IX+07)
         ADD HL,BC           ;update element size
         LD (IX+06),L
         LD (IX+07),H
         POP HL
         PUSH IX
         POP DE
         ADD HL,DE           ;calculate start point
         INC HL
         INC HL
         INC HL
         JP 1655,MAKE-ROOM

;        Find a string in the variables area

find-var LD (chardata),A     ;array/string letter
         LD BC,(CH-ADD)
         PUSH BC             ;save CH-ADD
         LD BC,chardata
         LD (CH-ADD),BC
         CALL 28B2,LOOK-VARS
         POP BC
         LD (CH-ADD),BC      ;restore CH-ADD
         PUSH HL
         POP IX              ;IX & HL contain location
         RET

;        Find input name in array
;        Game input/array = n$/z$
;        Pubs input/array = f$/x$


search-x LD A,78
         JR search
search-z LD A,7A
search   PUSH IY             ;save IY
         LD (save-IY),IY
         CALL find-var       ;find array
         LD DE,cursz
         LD BC,0008
         ADD HL,DE
         ADD HL,BC
         DEC HL
         ADD IX,BC
         XOR A
         LD (matched),A      ;reset match flag
         LD A,6E             ;code "n"
         LD IY,chardata
         BIT 1,(IY+00)
         JR NZ,is-z          ;if z$, jump
         RES 3,A             ;if x$, change code "n" to "f"
is-z     POP IY
         PUSH IY
         PUSH IX             ;initial array pointer
         PUSH HL             ;current array data size
         PUSH IX
         CALL find-var       ;find input string
         POP HL
         LD IY,chardata
         EXX
         LD BC,0000          ;clear element count (x$)
         EXX
         LD E,00
         BIT 3,(IY+00)
         JR Z,nmore2
         LD BC,offset
         ADD HL,BC           ;set start pointer for search
         JR nmore2
nextz    BIT 3,(IY+00)
         JR Z,noff           ;if x$, jump
         INC HL
         LD A,(HL)           ;= lname for z$ (packed)
         INC HL
         LD B,00
         LD C,(HL)           ;number of bytes from prev name in array
         PUSH HL
         INC HL              ;points to rest of name
         EX DE,HL
         LD HL,namedata
         ADD HL,BC           ;skip over duplicated part of name
         EX DE,HL
         LD C,A
         DEC C
         JR Z,nomore
         LDIR                ;copy rest of name to unpack area
nomore   POP HL
         ADD A,(HL)
         DEC A
         LD C,A              ;= lname for z$ (unpacked)
         LD (element),A      ;save lname temporarily
         PUSH BC
         CALL pad
         POP BC
         LD HL,namedata
         JR nboth
noff     LD C,(HL)
         DEC C               ;= lname for x$
         INC HL
nboth    LD B,(IX+01)        ;length of input string
         PUSH HL             ;start of name in array
         PUSH IX
         POP HL              ;start of input string
         INC HL
         INC HL
         INC HL
         JR nsamez
next     LD A,(HL)           ;next character from input string
         INC HL
         EX (SP),HL
         LD D,(HL)           ;next character from array
         INC HL
         EX (SP),HL
         DEC B
         DEC C
         CP D
         JR Z,nsamez
         BIT 3,(IY+00)
         JR Z,nmorez         ;if x$, jump
         JR C,done           ;if z$, check carry
nmorez   POP HL              ;discard pointer
         LD BC,0028
         LD HL,namedata
         LD DE,namedata+0028
         LDIR                ;copy current name to previous
nmore1   POP HL              ;get current array pointer
         LD E,(HL)           ;= ldata
nmore2   LD D,00
         ADD HL,DE           ;= array pointer + ldata
         POP DE              ;get current array data size
         PUSH DE
         PUSH HL             ;save new array pointer
         EXX
         INC BC              ;element count (just for x$)
         EXX
         LD A,H
         CP D
         JR C,nextz
         JR NZ,done1
         LD A,L
         CP E
         JR C,nextz
         JR done1
nsamez   XOR A
         CP C
         JR Z,zdone          ;end of array name?
         XOR A
         CP B
         JR NZ,next          ;end of input name?
         JR all-same
zdone    XOR A
         CP B
         JR NZ,nmorez
all-same LD A,1
         LD (matched),A      ;set match flag
         BIT 3,(IY+00)
         JR Z,done           ;jump if x$
         LD IY,(save-IY)
         LD A,65             ;code "e"
         CALL find-var
         LD BC,0006
         ADD HL,BC
         LD DE,namedata
         EX DE,HL
         LD BC,0028
         LDIR                ;copy expanded name to e$
         LD HL,element
         LD C,(HL)           ;length of expanded name
         CALL pad
done     POP HL              ;discard pointer
done1    POP HL              ;get current array pointer
         POP DE              ;discard array data size
         POP BC              ;get initial array pointer
         XOR A
         SBC HL,BC           ;calculate array subscript
         INC HL
         PUSH HL
         POP BC              ;return array subscript
         POP IY              ;restore IY
         EXX
         LD (element),BC     ;save element count
         EXX
         RET

;        find publisher name to match 'from' code

findfrom LD A,6A             ;code "j"
         CALL find-var
         LD DE,0004
         ADD HL,DE
         LD E,(HL)
         ADD HL,DE
         INC HL
         LD C,(HL)           ;= 'from' code
         LD B,00
         INC HL
         BIT 7,(HL)
         JR Z,not-set
         INC B               ;'from' code > 255
not-set  PUSH BC
         LD A,78             ;= code "x"
         CALL find-var
         POP BC
         LD DE,0008
         ADD HL,DE
         PUSH HL
         JR end-loop
bgn-loop LD E,(HL)
         ADD HL,DE
end-loop DEC BC
         LD A,B
         OR C
         JR NZ,bgn-loop
         POP DE
         INC HL
         XOR A
         SBC HL,DE
         PUSH HL
         POP BC              ;return array subscript
         RET

;        set up registers for tape routines

sa-ld-vf LD A,7A             ;code "z"
         CALL find-var
         LD E,(IX+01)        ;set DE to array length
         LD D,(IX+02)
         INC IX              ;step past array name/length
         INC IX
         INC IX              ;IX points to start of data
         LD A,FF             ;indicates 'data block'
         RET

;        save z$

save-z   CALL sa-ld-vf
         JP 04C2, SA-BYTES   ;save a data block

;        load z$

load-z   CALL sa-ld-vf
         PUSH IX
         POP HL              ;HL points to start of data
         LD IX,tapedata+03   ;IX points to start of 'header'
         JP 0839,LD-DATA+0B  ;load a data block

;        verify z$

verf-z   CALL sa-ld-vf
         AND A               ;reset carry (indicates 'verify')
         JP 0800,VR-CONT-3   ;verify a data block

;        calculate number of characters at start of new name
;        which match previous name in array

prefix   LD A,65             ;code "e"
         CALL find-var
         LD BC,0006
         ADD IX,BC
         LD C,00
         LD B,28
         LD HL,namedata+0028 ;point to previous name
nextchar LD A,(IX+00)
         LD E,(HL)
         CP E
         JR Z,nomatch
         INC C
         INC IX
         INC HL
         DJNZ nextchar
nomatch  LD B,00             ;return number of characters matched
         RET

;        tape error trap

err-trap LD BC,Deg           ;line number to go to
         LD (OLDPPC),BC
         LD A,FF
         LD (ERR-NR),A
         INC A
         LD (OSPCC),A
         LD HL,(E-LINE)
         CALL 1652,ONE-SPACE
         EX DE,HL
         LD (HL),E8          ;code "CONTINUE"
brk-loop CALL 1F54,BREAK-KEY
         JP C,12CF,MAIN-3
         JR brk-loop

;        Reclaim unused blocks in array

reclaimx LD A,78             ;code "x"
         JR reclaim
reclaimz LD A,7A             ;code "z"
reclaim  CALL find-var
         LD BC,ldiff         ;bytes to reclaim
         LD L,(IX+01)
         LD H,(IX+02)
         AND A               ;reset carry
         SBC HL,BC           ;reduce array size
         PUSH HL             ;save array size
         LD (IX+01),L
         LD (IX+02),H
         LD L,(IX+06)
         LD H,(IX+07)
         SBC HL,BC           ;reduce element size
         LD (IX+06),L
         LD (IX+07),H
         POP HL
         PUSH IX
         POP DE
         ADD HL,DE           ;calculate start point
         INC HL
         INC HL
         INC HL
         JP 19E8,RECLAIM-2

;        Pad work space with nulls after text of name

pad      LD A,28
         SUB C               ;calulate bytes to pad
         RET Z               ;return if none (lname is 40)
         LD HL,namedata
         ADD HL,BC
         LD (HL),00          ;set first spare byte to null
         DEC A
         RET Z               ;return if only one
         LD D,H
         LD E,L
         INC DE
         LD C,A
         LDIR                ;copy nulls to remaining bytes
         RET

************************************************************************


*NOTES ON STRING NAMES*

 a$      READ variable
 b$()    32 blanks
 c$      current data block letter
 d$      initial letter of current name
 e$()    expanded name
 f$      name of company
 g$      game type
 h$      entry code
 i$      run options
 j$      current record from z$()
 k$      saved input name
 l$      subroutine variable
 m$      magazine code
 n$      name of game
 o$      option response
 p$      file name prefix
 q$()    sorted list for query
 r$      query item
 s$      saved run option
 t$      run option description
 u$      input field delimiter
 v$      file name
 w$      delete code
 x$()    data block (company names)
 y$      abbreviated month names
 z$()    data block (game details)


************************************************************************

